from socket import socket

import pytest


@pytest.fixture
def anyio_backend():
    return "asyncio"


@pytest.fixture()
def unused_tcp_port() -> int:
    with socket() as sock:
        sock.bind(("127.0.0.1", 0))
        return sock.getsockname()[1]



import sys
from pathlib import Path
from uuid import uuid4

import easywebdav  # type: ignore
import pytest
from anyio import to_thread
from fps import get_root_module, merge_config

CONFIG = {
    "jupyverse": {
        "type": "jupyverse_api.main:JupyverseModule",
        "modules": {
            "app": {
                "type": "jupyverse_api.main:AppModule",
            },
            "webdav": {
                "type": "webdav",
            },
        }
    }
}


@pytest.mark.anyio
@pytest.mark.skipif(sys.version_info < (3, 10), reason="Requires Python >=3.10")
async def test_webdav(unused_tcp_port):
    config = merge_config(
        CONFIG,
        {
            "jupyverse": {
                "config": {
                    "port": unused_tcp_port,
                },
                "modules": {
                    "webdav": {
                        "config": {
                            "account_mapping": [{"username": "foo", "password": "bar"}],
                        }
                    }
                }
            }
        }
    )
    async with get_root_module(config):
        webdav = easywebdav.connect(
            "127.0.0.1", port=unused_tcp_port, path="webdav", username="foo", password="bar"
        )
        dirname = str(uuid4())
        assert not Path(dirname).exists()
        await to_thread.run_sync(webdav.mkdir, dirname)
        assert Path(dirname).is_dir()
        await to_thread.run_sync(webdav.rmdir, dirname)
        assert not Path(dirname).exists()



from uuid import uuid4

from pydantic import BaseModel


class Account(BaseModel):
    username: str = uuid4().hex
    password: str = uuid4().hex
    permissions: list[str] = ["+"]


class Provider(BaseModel):
    prefix: str = "/webdav"
    uri: str = "file://."


class WebDAVConfig(BaseModel):
    account_mapping: list[Account] = [Account()]
    provider_mapping: list[Provider] = [Provider()]



from fps import Module

from jupyverse_api.app import App

from .config import WebDAVConfig
from .routes import WebDAV


class WebDAVModule(Module):
    def __init__(self, name: str, **kwargs):
        super().__init__(name)
        self.config = WebDAVConfig(**kwargs)

    async def prepare(self) -> None:
        app = await self.get(App)

        webdav = WebDAV(app, self.config)
        self.put(webdav)
